from pathlib import Path

def sha256_file_hex(path: Path, chunk_size: int = ...) -> str: ...
def render_single_page_pdf(text: str) -> bytes: ...
def parse_extracted_frame_numbers(paths: list[str]) -> list[int]: ...
def build_frame_records(
    paths: list[str], *, relative_to: str | None = ..., zero_based: bool = ...
) -> list[tuple[int, str]]: ...
def build_expected_frame_records(
    frame_count: int, ext: str = ...
) -> list[tuple[int, str]]: ...
