# Generated by Django 6.0 on 2026-03-31 07:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("endoreg_db", "0013_remove_legacy_requirement_models"),
    ]

    operations = [
        migrations.AddField(
            model_name="sensitivemeta",
            name="tags",
            field=models.ManyToManyField(
                blank=True, help_text="Validation tags", to="endoreg_db.tag"
            ),
        ),
        migrations.AddField(
            model_name="sensitivemeta",
            name="validation_comment",
            field=models.TextField(blank=True, default=""),
        ),
    ]
